/******************************************************************************
*   File:       xeno_astropci.h
*   Author:     Marco Bonati, modified by Scott Streit and Michael Ashley
*   Abstract:   Linux device driver for the SDSU PCI Interface Board. This file
*               contains the initialization and all user defined functions for
*               the driver.
*
*
*   Revision History:     Date      Who   Version    Description
*   --------------------------------------------------------------------------
*                       06/16/00    sds     1.3      Removed misc defines no
*                                                    longer used by the driver.
*                       07/11/00    sds     1.4      Added function prototype
*                                                    for astropci_read_reply().
*                       01/26/01    sds     1.4      Just changed some values.
*                       07/13/01    sds     1.6      Removed a lot of code to
*                                                    support version 1.6.
*                       09-Jan-2003 mcba    1.7B     Removed lots more code.
*   Development notes:
*   -----------------
*   This driver has been tested on Redhat Linux 7.2 with Kernel 2.4.20.
******************************************************************************/
#ifndef _ASTROPCI_RTAI_H
#define _ASTROPCI_RTAI_H

/******************************************************************************
        General Definitions
******************************************************************************/
#define DRIVER_NAME		(char *)"xeno_astropci"
#define DRIVER_VERSION		(char *)"1.7 Jan 13, 2004"
#define SUPPORTED_KERNELS	(char *)"2.4.x (Redhat 7.2, 9), 2.6.5-1.358 (Fedora Core2)"
#define MAX_DEV                 4   /* Max number of devices */
#define ARC_PCI_DEVICE_ID      0x1801
#define REGS_SIZE              (0x9C/sizeof(uint32_t))*sizeof(uint32_t)
#define DEBUG_OFF                
#define DEBUG_ASTROPCI          1
#define CMD_MAX			6

#define INPUT_FIFO  0  /* For astropci_wait_for_condition() */
#define OUTPUT_FIFO 1
#define CHECK_REPLY 2


/*********************************************
        Information retrieval commands
*********************************************/
#define ASTROPCI_GET_HCTR               0x1
#define ASTROPCI_GET_PROGRESS           0x2
#define ASTROPCI_GET_DMA_ADDR           0x3
#define ASTROPCI_GET_HSTR               0x4
/* Commands 0x5 and 0x6 are used by WIN2K */
#define ASTROPCI_GET_FRAMES_READ	0x7

/*********************************************
        Information set commands
*********************************************/
#define ASTROPCI_HCVR_DATA              0x10
#define ASTROPCI_SET_HCTR               0x11
#define ASTROPCI_SET_HCVR               0x12
#define ASTROPCI_PCI_DOWNLOAD           0x13
#define ASTROPCI_PCI_DOWNLOAD_WAIT      0x14
#define ASTROPCI_COMMAND                0x15
#define ASTROPCI_MUNMAP                 0x16

/*********************************************

        Generic Constants
*********************************************/
#define DEBUG                           0
#define REPLY_BUFFER_EMPTY              -1

/*********************************************
        DSP Fifo Check Constants
*********************************************/
#define INPUT_FIFO_OK_MASK              0x00000002
#define OUTPUT_FIFO_OK_MASK             0x00000004
#define HTF_BIT_MASK                    0x00000038
#define BUSY_WAIT_DELAY                 100                     /* usec */
#define SLEEP_WAIT_DELAY                10000                   /* usec */
#define BUSY_MAX_WAIT                   1000                    /* usec */
#define SLEEP_MAX_WAIT                  800000                  /* usec */

/*********************************************
        Vector Commands
*********************************************/
#define READ_REPLY_VALUE                0x83
#define CLEAR_REPLY_FLAGS               0x85
#define WRITE_PCI_ADDRESS               0x8D
#define CLEAR_INTERRUPT			0x8073
#define READ_PCI_IMAGE_ADDR             0x8075
#define ABORT_READOUT                   0x8079
#define READ_NUMBER_OF_FRAMES_READ	0x807D
#define PCI_DOWNLOAD                    0x802F
#define WRITE_COMMAND                   0xB1

/*********************************************
        DSP Replies
*********************************************/
#define DON                             0x00444F4E
#define RDR                             0x00524452
#define ERR                             0x00455252
#define SYR                             0x00535952
#define TIMEOUT                         0x544F5554
#define READOUT                         0x524F5554

/*********************************************
        Interrupt constants
*********************************************/
#define REPLY_INTERRUPT                 0x8
#define IMAGE_BUFFER1_INTERRUPT         0x10
#define IMAGE_BUFFER2_INTERRUPT         0x20
#define CLEAR_REPLY_INTERRUPT           0x8073
#define CLEAR_IMAGE_BUFF1_INTERRUPT     0x8075
#define CLEAR_IMAGE_BUFF2_INTERRUPT     0x8077
#define DMA_INTERRUPTING                0x40
#define ABORT_INTERRUPT                 0x78
#define REPLY_DATA_EXISTS               0x4

/*********************************************
        DMA constants
*********************************************/
#define DMA_INTERRUPTING                0x40
#define IMAGE_BUFF1_BUSY                0x10
#define IMAGE_BUFF1_CLEAR               0xFFFFFFEF
#define IMAGE_BUFF2_BUSY                0x20
#define IMAGE_BUFF2_CLEAR               0xFFFFFFDF
#define ASTROPCI_DEVICE_ID              0x1801
#define ASTROPCI_VENDOR_ID              0x1057

/******************************************************************************
        Debug Print Definitions
******************************************************************************/
#ifdef DEBUG_ASTROPCI
        #define PDEBUG(fmt, args...) printk (KERN_WARNING fmt, ## args)
#else
        #define PDEBUG(fmt, args...)
#endif

#define PPDEBUG(fmt, args...)
#define astropci_printf(fmt, args...) printk("<1>astropci: " fmt, ## args)

/******************************************************************************
        PCI DSP Control Registers
******************************************************************************/
#define HCTR		0x10	/* Host interface control register      */
#define HSTR		0x14	/* Host interface status register       */
#define HCVR		0x18	/* Host command vector register         */
#define REPLY_BUFFER	0x1C	/* Reply Buffer				*/
#define CMD_DATA	0x20	/* DSP command register                 */


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
  #define writel(value, addr) writel(value, (unsigned long *)(addr)) 
  #define readl(addr) readl((unsigned long *)(addr))
  #define readw(addr) readw((unsigned long *)(addr))
  #define writew(value, addr) writew(value,(unsigned long *)(addr))
#endif



/*************************************************************************** 
        State Structure - Driver state structure. All state variables related
                                  to the state of the driver are kept in here.
***************************************************************************/
typedef struct astropci_state {
        struct pci_dev *pdev;
        long           ioaddr;
        RT_INTR        intr_desc;
      //RT_HEAP        heap_desc;
        RT_SEM         i_pulse;
        void           *shmem;
        uint8_t        have_irq;
        uint32_t       imageBufferSize;
        uint32_t       imageBufferStart;
        short          opened;
} astropci_t;

/******************************************************************************
        Prototypes for main entry points
******************************************************************************/
static int astropci_open (struct inode *, struct file *);
static int astropci_close (struct inode *, struct file *);
static int astropci_ioctl (struct inode *, struct file *, unsigned int, unsigned long);
static int astropci_mmap (struct file *file, struct vm_area_struct *vma);

/******************************************************************************
        Prototypes for other functions
******************************************************************************/
static int __devinit astropci_probe (struct pci_dev *pdev, const struct pci_device_id *ent);
static void __devexit astropci_remove (struct pci_dev *pdev);
static int __astropci_isr (xnintr_t *cookie);
static int astropci_flush_reply_buffer (int dev);
static int astropci_check_reply_flags (int dev);
static int astropci_check_dsp_input_fifo (int dev);
static int astropci_check_dsp_output_fifo (int dev);
static int astropci_wait_for_condition (int dev, int condition_to_wait_for);
static int astrocpi_check_for_readout (int dev);
static int astropci_set_buffer_addresses (int dev);


#endif
